@@ -30,6 +30,7 @@
                   *allow-with-interrupts*
                   *interrupts-enabled*
                   *interrupt-pending*
+                  #!+sb-thruption *thruption-pending*
                   *type-system-initialized*))
 
 (defvar *cold-init-complete-p*)
@@ -201,7 +202,7 @@ any non-negative real number."
            :format-arguments (list seconds)
            :datum seconds
            :expected-type '(real 0)))
-  #!-win32
+  #!-(and win32 (not sb-thread))
   (multiple-value-bind (sec nsec)
       (if (integerp seconds)
           (values seconds 0)
@@ -215,7 +216,7 @@ any non-negative real number."
           do (decf sec (expt 10 8))
              (sb!unix:nanosleep (expt 10 8) 0))
     (sb!unix:nanosleep sec nsec))
-  #!+win32
+  #!+(and win32 (not sb-thread))
   (sb!win32:millisleep (truncate (* seconds 1000)))
   nil)
 
