@@ -575,7 +575,7 @@
     (emit-label trampoline-label)
     (popw rbp-tn (frame-word-offset return-pc-save-offset)))
   (when alignp
-    (emit-alignment n-lowtag-bits #x90))
+    (emit-alignment n-lowtag-bits :long-nop))
   (emit-label start-label))
 
 ;;; Non-TR local call for a fixed number of values passed according to
