@@ -138,7 +138,9 @@
 
 (defenum (:start 24)
   object-not-list-trap
-  object-not-instance-trap)
+  object-not-instance-trap
+  #!+sb-safepoint global-safepoint-trap
+  #!+sb-safepoint csp-safepoint-trap)
 
 ;;;; static symbols
 
