@@ -186,7 +186,6 @@
 
 #!+win32
 (progn
-
   (def!constant read-only-space-start #x22000000)
   (def!constant read-only-space-end   #x220ff000)
 
@@ -306,7 +305,9 @@
 
 (defenum (:start 24)
   object-not-list-trap
-  object-not-instance-trap)
+  object-not-instance-trap
+  #!+sb-safepoint global-safepoint-trap
+  #!+sb-safepoint csp-safepoint-trap)
 
 ;;;; static symbols
 
@@ -382,3 +383,6 @@
 ;;; FIXME: Is this used? Delete it or document it.
 ;;; cf the sparc PARMS.LISP
 (defparameter *assembly-unit-length* 8)
+
+#!+win32
+(defconstant +win32-tib-arbitrary-field-offset+ #.(+ #xE10 (* 4 63)))
