@@ -21,6 +21,8 @@
 #define PAGE_BYTES BACKEND_PAGE_BYTES
 
 typedef signed long page_index_t;
+#define PAGE_INDEX_FMT "ld"
+
 typedef signed char generation_index_t;
 
 extern void gc_init(void);
@@ -38,6 +40,6 @@ extern void clear_auto_gc_trigger(void);
 
 extern boolean maybe_gc(os_context_t *context);
 
-extern unsigned long bytes_consed_between_gcs;
+extern os_vm_size_t bytes_consed_between_gcs;
 
 #endif /* _GC_H_ */
