@@ -18,7 +18,6 @@
 #include "os.h"
 #include "arch.h"
 #include "lispregs.h"
-#include "signal.h"
 #include "alloc.h"
 #include "interrupt.h"
 #include "interr.h"
@@ -78,7 +77,7 @@ context_eflags_addr(os_context_t *context)
 #elif defined __NetBSD__
     return &(context->uc_mcontext.__gregs[_REG_EFL]);
 #elif defined LISP_FEATURE_WIN32
-    return (int *)&context->EFlags;
+    return (int *)&context->win32_context->EFlags;
 #else
 #error unsupported OS
 #endif
@@ -113,6 +112,10 @@ void arch_skip_instruction(os_context_t *context)
         case trap_FunEndBreakpoint: /* not tested */
             break;
 
+#ifdef LISP_FEATURE_SB_SAFEPOINT
+        case trap_GlobalSafepoint:
+        case trap_CspSafepoint:
+#endif
         case trap_PendingInterrupt:
         case trap_Halt:
         case trap_SingleStepAround:
