@@ -122,6 +122,7 @@
 (with-test (:name :bug-994487)
   (let ((f (compile nil `(lambda (char)
                            (code-char (1+ (char-code char)))))))
-    (assert (equal `(function (t) (values (sb-kernel:character-set ((1 . 1114111)))
+    (assert (equal `(function (t) (values (sb-kernel:character-set
+                                           ((1 . ,(1- char-code-limit))))
                                           &optional))
                    (sb-impl::%fun-type f)))))
